#!/usr/bin/perl

my $min_mem = 8;	# Minimum memory block size (KB)
my $max_mem = 65536;	# Maximum memory block size (KB), must fit in physical memory
my @item_sizes = (16,64,128,1024,4096);	# Sizes of items
my @randomized = (0,1);	# Try randomized accesses?
my @modify = (0,1);	# Try read-write accesses?
my $measure_ms = 1000;	# Duration of measurement in ms
my $array = 0;		# Items are accessed as an array instead of a list
my $huge = 0;		# Use huge pages (hugetlbfs required)

if (defined $ARGV[0]) {	# First argument: name of a directory to store the results to
	my $dir = $ARGV[0];
	-d $dir or mkdir $dir or die "Cannot create $dir";
	chdir $dir or die;
	-f "access.c" or symlink "../access.c", "access.c" or die;
	-f "Makefile" or symlink "../Makefile", "Makefile" or die;
}

my $machine = `hostname`;
chomp $machine;
$machine .= " Array" if $array;
$machine .= " HugePages" if $huge;

open GP, ">plot.gp" or die;
print GP <<EOF ;
set terminal png large size 1024,768
set logscale x 2
set key left
set xlabel "data set size [KB]"
set ylabel "time per access [ns]"
set grid
set yrange [1:120]	# Change if necessary

EOF

for my $r (@randomized) {
	for my $m (@modify) {
		my @plots = ();
		for my $s (@item_sizes) {
			my $f = "a-$r-$m-$s.log";
			push @plots, "'$f' title '${s}B items' with linespoints";
			next if -f $f;
			my @o = ( "-DSIZE=$s", "-DMS=$measure_ms" );
			push @o, "-DRANDOMIZE" if $r;
			push @o, "-DMODIFY" if $m;
			push @o, "-DARRAY" if $array;
			push @o, "-DHUGE" if $huge;
			my $o = join(" ", @o);
			`rm -f access`; die if $?;
			`make access F="$o"`; die if $?;
			open D, ">$f" or die;
			for (my $mem=$min_mem; $mem<=$max_mem; $mem*=2) {
				my $t = `./access $mem`; die if $?;
				chomp $t;
				print "$mem ($o): $t\n";
				print D "$mem\t$t\n";
			}
			close D;
		}

		print GP "set output 'a-$r-$m.png'\n";
		my $title = "Timing of " . ($r ? "Random" : "Sequential") . " " . ($m ? "Read-Write" : "Read-Only") . " Accesses ($machine)";
		print GP "set title '$title'\n";
		print GP "plot ", join(", ", @plots), "\n\n";
	}
}

for my $s (@item_sizes) {
	print GP "set output 'a-size-$s.png'\n";
	print GP "set title 'Item size $s ($machine)'\n";
	my @a = ();
	for my $r (@randomized) {
		for my $m (@modify) {
			my $t = ($r ? "Random" : "Sequential") . " " . ($m ? "R+W" : "R");
			push @a, "'a-$r-$m-$s.log' title '$t' with linespoints";
		}
	}
	print GP "plot ", join(", ", @a), "\n\n";
}

close GP;
